@@ -27,6 +27,7 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
+#include <signal.h>
 
 #include <jack/internal.h>
 #include <jack/engine.h>
@@ -142,7 +143,17 @@ jack_remove_client (jack_engine_t *engine, jack_client_internal_t *client)
 	/* ignore the driver, which counts as a client. */
 
 	if (engine->temporary && (jack_slist_length(engine->clients) <= 1)) {
-		exit (0);
+		if (engine->wait_pid >= 0) {
+			/* tell the waiter we're done
+			   to initiate a normal shutdown.
+			*/
+			VERBOSE (engine, "Kill wait pid to stop");
+			kill (engine->wait_pid, SIGUSR2);
+			sleep (-1);
+		} else {
+			exit (0);
+		}
+
 	}
 }
 
