@@ -49,7 +49,9 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 #include <net_driver.h>
 #include <netjack_packet.h>
+#if HAVE_SAMPLERATE
 #include <samplerate.h>
+#endif
 
 #if HAVE_CELT
 #include <celt/celt.h>
@@ -67,6 +69,7 @@ JSList *playback_srcs = NULL;
 int playback_channels = 0;
 int playback_channels_audio = 2;
 int playback_channels_midi = 1;
+int dont_htonl_floats = 0;
 
 int latency = 5;
 jack_nframes_t factor = 1;
@@ -125,7 +128,9 @@ alloc_ports (int n_capture_audio, int n_playback_audio, int n_capture_midi, int
 	    capture_srcs = jack_slist_append(capture_srcs, celt_decoder_create( celt_mode ) );
 #endif
 	} else {
+#if HAVE_SAMPLERATE
 	    capture_srcs = jack_slist_append (capture_srcs, src_new (SRC_LINEAR, 1, NULL));
+#endif
 	}
         capture_ports = jack_slist_append (capture_ports, port);
     }
@@ -162,7 +167,9 @@ alloc_ports (int n_capture_audio, int n_playback_audio, int n_capture_midi, int
 	    playback_srcs = jack_slist_append(playback_srcs, celt_encoder_create( celt_mode ) );
 #endif
 	} else {
+#if HAVE_SAMPLERATE
 	    playback_srcs = jack_slist_append (playback_srcs, src_new (SRC_LINEAR, 1, NULL));
+#endif
 	}
 	playback_ports = jack_slist_append (playback_ports, port);
     }
@@ -256,7 +263,8 @@ process (jack_nframes_t nframes, void *arg)
     packet_bufX = packet_buf + sizeof (jacknet_packet_header) / sizeof (jack_default_audio_sample_t);
 
     /* ---------- Send ---------- */
-    render_jack_ports_to_payload (bitdepth, playback_ports, playback_srcs, nframes, packet_bufX, net_period);
+    render_jack_ports_to_payload (bitdepth, playback_ports, playback_srcs, nframes, 
+	    packet_bufX, net_period, dont_htonl_floats);
 
     /* fill in packet hdr */
     pkthdr->transport_state = jack_transport_query (client, &local_trans_pos);
@@ -339,7 +347,8 @@ process (jack_nframes_t nframes, void *arg)
             //printf("Frame %d  \tRecovered from dropouts\n", framecnt);
             cont_miss = 0;
         }
-        render_payload_to_jack_ports (bitdepth, packet_bufX, net_period, capture_ports, capture_srcs, nframes);
+        render_payload_to_jack_ports (bitdepth, packet_bufX, net_period, 
+		capture_ports, capture_srcs, nframes, dont_htonl_floats);
 
 	state_currentframe = framecnt;
 	state_recv_packet_queue_time = recv_time_offset;
@@ -465,7 +474,7 @@ main (int argc, char *argv[])
     sprintf(client_name, "netsource");
     sprintf(peer_ip, "localhost");
 
-    while ((c = getopt (argc, argv, ":R:n:s:h:p:C:P:i:o:l:r:f:b:m:c:")) != -1)
+    while ((c = getopt (argc, argv, ":H:R:n:s:h:p:C:P:i:o:l:r:f:b:m:c:")) != -1)
     {
         switch (c)
         {
@@ -526,6 +535,9 @@ main (int argc, char *argv[])
             case 'R':
             redundancy = atoi (optarg);
             break;
+            case 'H':
+            dont_htonl_floats = atoi (optarg);
+            break;
             case ':':
             fprintf (stderr, "Option -%c requires an operand\n", optopt);
             errflg++;
